#include "Arduino.h"
#include <avr/pgmspace.h>
#include "OLED.h"

void OLED::delay_ms(unsigned int ms)
{
	unsigned int a;
	while(ms)
	{a=3000;while(a--);ms--;}
}

void OLED::IIC_Start()
{
	OLED_SCLK_Set();
	OLED_SDIN_Set();
	OLED_SDIN_Clr();
	OLED_SCLK_Clr();
}

void OLED::IIC_Stop()
{
	OLED_SCLK_Clr();
	OLED_SDIN_Clr();
	OLED_SDIN_Set();
	OLED_SCLK_Set() ;
}

void OLED::Write_IIC_Byte(unsigned char IIC_Byte)
{
	unsigned char i;
	unsigned char m,date;
	date=IIC_Byte;
	for(i=0;i<8;i++)		
	{
		m=date;
		OLED_SCLK_Clr();
		m=m&0x80;
		if(m==0x80)
			{OLED_SDIN_Set();}
		else
			OLED_SDIN_Clr();
		date=date<<1;
		OLED_SCLK_Set();
	}
	OLED_SCLK_Clr();
	OLED_SCLK_Set() ;
}

void OLED::OLED_WR_Byte(unsigned dat,unsigned mode)
{
	IIC_Start();
	Write_IIC_Byte(0x78);		
	Write_IIC_Byte(mode);
	Write_IIC_Byte(dat);
	IIC_Stop();
}

void OLED::OLED_Set_Pos(unsigned char x, unsigned char y) 
{
	OLED_WR_Byte(0xb0+y,OLED_CMD);
	OLED_WR_Byte((x&0x0f),OLED_CMD);
	OLED_WR_Byte(((x&0xf0)>>4)|0x10,OLED_CMD);
}   	  
				    
void OLED::OLED_Init(void)
{ 	
	delay_ms(100);
	OLED_WR_Byte(0xae,OLED_CMD);//--turn off oled panel
	OLED_WR_Byte(0x00,OLED_CMD);//---set low column address
	OLED_WR_Byte(0x10,OLED_CMD);//---set high column address
	OLED_WR_Byte(0x40,OLED_CMD);//--set start line address  Set Mapping RAM Display Start Line (0x00~0x3F)
	OLED_WR_Byte(0x81,OLED_CMD);//--set contrast control register
	OLED_WR_Byte(Brightness,OLED_CMD); // Set SEG Output Current Brightness
	OLED_WR_Byte(0xa1,OLED_CMD);//--Set SEG/Column Mapping     0xa0左右反置 0xa1正常
	OLED_WR_Byte(0xc8,OLED_CMD);//Set COM/Row Scan Direction   0xc0上下反置 0xc8正常
	OLED_WR_Byte(0xa6,OLED_CMD);//--set normal display
	OLED_WR_Byte(0xa8,OLED_CMD);//--set multiplex ratio(1 to 64)
	OLED_WR_Byte(0x3f,OLED_CMD);//--1/64 duty
	OLED_WR_Byte(0xd3,OLED_CMD);//-set display offset	Shift Mapping RAM Counter (0x00~0x3F)
	OLED_WR_Byte(0x00,OLED_CMD);//-not offset
	OLED_WR_Byte(0xd5,OLED_CMD);//--set display clock divide ratio/oscillator frequency
	OLED_WR_Byte(0x80,OLED_CMD);//--set divide ratio, Set Clock as 100 Frames/Sec
	OLED_WR_Byte(0xd9,OLED_CMD);//--set pre-charge period
	OLED_WR_Byte(0xf1,OLED_CMD);//Set Pre-Charge as 15 Clocks & Discharge as 1 Clock
	OLED_WR_Byte(0xda,OLED_CMD);//--set com pins hardware configuration
	OLED_WR_Byte(0x12,OLED_CMD);
	OLED_WR_Byte(0xdb,OLED_CMD);//--set vcomh
	OLED_WR_Byte(0x40,OLED_CMD);//Set VCOM Deselect Level
	OLED_WR_Byte(0x20,OLED_CMD);//-Set Page Addressing Mode (0x00/0x01/0x02)
	OLED_WR_Byte(0x02,OLED_CMD);//
	OLED_WR_Byte(0x8d,OLED_CMD);//--set Charge Pump enable/disable
	OLED_WR_Byte(0x14,OLED_CMD);//--set(0x10) disable
	OLED_WR_Byte(0xa4,OLED_CMD);// Disable Entire Display On (0xa4/0xa5)
	OLED_WR_Byte(0xa6,OLED_CMD);// Disable Inverse Display On (0xa6/a7) 
	OLED_WR_Byte(0xaf,OLED_CMD);//--turn on oled panel
	fill(0);	//清屏
}  
         
void OLED::showChar(unsigned char x, unsigned char y, char data)
{       
  unsigned char c=0,i=0;
  c=data-' ';//获取偏移量
  if(x>=Max_Column/8){x=0;y++;}
  OLED_Set_Pos(x*8,y*2);  
  for(i=0;i<8;i++)
    OLED_WR_Byte(pgm_read_byte_near(&F8X16[c*16+i]),OLED_DATA);
  OLED_Set_Pos(x*8,y*2+1);
  for(i=0;i<8;i++)
    OLED_WR_Byte(pgm_read_byte_near(&F8X16[c*16+i+8]),OLED_DATA); 
}

void OLED::showChinese(unsigned char x, unsigned char y, char *string)
{               
  unsigned char c,i;

  while(pgm_read_byte_near(&F16X16[c].Idx[0]) != ' ')
  {
    if(pgm_read_byte_near(&F16X16[c].Idx[0]) == (unsigned char)string[0]
    && pgm_read_byte_near(&F16X16[c].Idx[1]) == (unsigned char)string[1]
    && pgm_read_byte_near(&F16X16[c].Idx[2]) == (unsigned char)string[2])
    {
      break;
    }
    c++;
  }
  
  OLED_Set_Pos(x*8,y*2);
  for(i=0;i<16;i++)
  {
    OLED_WR_Byte(pgm_read_byte_near(&F16X16[c].Hzk1[i]),OLED_DATA);
  } 
  OLED_Set_Pos(x*8,y*2+1);  
  for(i=0;i<16;i++)
  { 
    OLED_WR_Byte(pgm_read_byte_near(&F16X16[c].Hzk2[i]),OLED_DATA);
  }   
}

void OLED::Int_to_Str(int temp ,char str[] ,char len)
{
    char i;
  for(i= 0; i<len; i++)
        str[i]=' ';

    if( temp >= 0 )
    {
        do
        {
            str[len-1]=temp%10+'0';
            temp/=10;
            len--;
            if( len == 0 )
                return;
        }while(temp);
    }
    else
    {
        temp = -temp;
        do
        {
            str[len-1]=temp%10+'0';
            temp/=10;
            len--;
            if( len == 0 )
                return;
        }while(temp);
        str[len-1] = '-';
    }
}

void OLED::Float_to_Str (float f_temp, char str[], char len, char pot )
{
    char i;
  for(i= 0; i<len; i++)
            str[i]=' ';
    if( f_temp >= 0 )
    {
        char i=0;
        int temp;
        for(i=0;i<pot;i++)
            f_temp = 10*f_temp;
        temp=(int)f_temp;
        i=0;
        do
        {
            str[len-1]=temp%10+'0';
            temp/=10;
            len--;
            if( len == 0 )
                return;
            i++;
            if( i == pot )
            {
                str[len-1] = '.';
                len--;
                if( len == 0 )
                    return;
            }            
        }while(temp);
        if( i < pot )
        {
            while( i < pot )
            {
                str[len-1] = '0';
                    len--;
                if( len == 0 )
                    return;
                i++;
            }
            str[len-1] = '.';
            len--;
            if( len == 0 )
                return;
            str[len-1] = '0';
            len--;
            if( len == 0 )
                return;            
        }
        else if( i == pot )
        {
            str[len-1] = '0';
            len--;
            if( len == 0 )
                return;
        }
    }
    else
    {
        char i=0;
        int temp;
        f_temp = -f_temp;
    for(i=0;i<pot;i++)
            f_temp = 10*f_temp;
        temp=(int)f_temp;
        i=0;
        do
        {
            str[len-1]=temp%10+'0';
            temp/=10;
            len--;
            if( len == 0 )
                return;
            i++;
            if( i == pot )
            {
                str[len-1] = '.';
                len--;
                if( len == 0 )
                    return;
            }            
        }while(temp);
        if( i < pot )
        {
            while( i < pot )
            {
                str[len-1] = '0';
                    len--;
                if( len == 0 )
                    return;
                i++;
            }
            str[len-1] = '.';
            len--;
            if( len == 0 )
                return;
            str[len-1] = '0';
            len--;
            if( len == 0 )
                return;            
        }
        else if( i == pot )
        {
            str[len-1] = '0';
            len--;
            if( len == 0 )
                return;
        }
        str[len-1] = '-';
    }
}

OLED::OLED(unsigned char pin1, unsigned char pin2)
{
	pin_scl = pin1;
	pin_sda = pin2;
	pinMode(pin_scl, OUTPUT);
	pinMode(pin_sda, OUTPUT);
	OLED_Init();
}
 
void OLED::displayOn(void)
{
	OLED_WR_Byte(0X8D,OLED_CMD);  //SET DCDC����
	OLED_WR_Byte(0X14,OLED_CMD);  //DCDC ON
	OLED_WR_Byte(0XAF,OLED_CMD);  //DISPLAY ON
}
   
void OLED::displayOff(void)
{
	OLED_WR_Byte(0X8D,OLED_CMD);  //SET DCDC����
	OLED_WR_Byte(0X10,OLED_CMD);  //DCDC OFF
	OLED_WR_Byte(0XAE,OLED_CMD);  //DISPLAY OFF
}		   			 

void OLED::fill(unsigned char fill_Data)
{
	unsigned char m,n;
	for(m=0;m<Max_Row/8;m++)
	{
		OLED_Set_Pos(0,m);
		for(n=0;n<Max_Column;n++)
			OLED_WR_Byte(fill_Data,OLED_DATA);
	}
}

void OLED::showString(unsigned char x, unsigned char y, char *string)
{
	unsigned char j=0;
	while(string[j]!='\0')
	{
		if((unsigned char)string[j] < 127)
		{
			showChar(x,y,string[j]);
			x++;
			if(x>=Max_Column/8){x=0;y++;}
			j++;
		}
		else
		{
      if(x>=Max_Column/8-1){x=0;y++;}
			showChinese(x,y,string+j);
			x+=2;
			if(x>=Max_Column/8){x=0;y++;}
			j+=3;
		}
	}
}

void OLED::showInt(unsigned char x, unsigned char y, int date, char len)
{
  char str[len+1];
  Int_to_Str(date, str, len);
  str[len] = 0;
  showString(x,y,str);
}

void OLED::showFloat(unsigned char x, unsigned char y, float date, char len, char pot)
{
  char str[len+1];
  Float_to_Str(date, str, len, pot);
  str[len] = 0;
  showString(x,y,str);
}

void OLED::drawBMP()
{ 	
	unsigned int j=0;
	unsigned char x,y;

	for(y=0;y<8;y++)
	{
		OLED_Set_Pos(0,y);
		for(x=0;x<128;x++)    
			OLED_WR_Byte(pgm_read_byte_near(&(BMP_Show[j++])),OLED_DATA);	    	
	}
}

